;*****************************************************************************
;*** Name of the program:		Remote-Control
;*** Function:             	Implementaion of I2C-Bus with connection to a
;***					   			DSM0822a - Display
;***									Program changes display about every 3 seconds
;*** Author:	Andreas Hesse
;*** Date of construction:  	2003-11-11
;*** Date of last revision: 	2003-11-16
;*** Version:               	1.0.0.0
;*** Target:						AT90Sxxxx (any AVR device)
;*** Comment:
;***
;***	Abbreviations:
;***	1. RI = Remote Input
;***	2. RO = Remote Output
;******************************************************************************
;***
;*** Version management:
;*** V 1.0.0.0  First construction based on  Version 1.0 of
;***
;***************************************************************************

;*** Includes



.include "..\..\AVR\ASM\Def\2313def.inc"		; change if an other device is used


.equ	TWIrd			= 1
.equ	TWIwr			= 0
.equ	I2CDispAdr	= 112

;******************************************************************************
;***** Registers
; Register lowhalf
.def	EEdrd			= r00 ;result data byte
.def  I2CIO1		= r01
;.def         	=r02
;.def         	=r03
;.def       	=r04
.def	SpecialSig1	= r05
				; first Byte of special signs of display
.def	SpecialSig2	= r06
				; second Byte of special signs of display
;.def      	=r07
;.def         	=r08
;.def         	=r09
;.def         	=r10
;.def         	=r11
;.def         	=r12
;.def         	=r13		; also: tBCD0 in Math
.def  actualRAM  	= r14	; actual value of RAM
								;  also: tBCD1 in Math
.def  errorsave	= r15
								; also: tBCD2 in Math

; Register highhalf
.def	tmp			= r16		;
.def	tmpdelay		= r17		; Used as: I2C Delay loop variable
.def	tmpdata		= r18		; Used as: I2C data transfer register
.def	tmpadr		= r19		; Used as: I2C address and direction register
.def	tmpstatus	= r20		; Used as: I2C bus status register
.def	tmpcnt		= r21
.def	tmperror		= r22
;.def	tmperror		= r23
.def	tmp16cntl	= r24
.def	tmp16cnth	= r25


;**** Interrupt Vectors ****

rjmp	MAIN			;Reset handle
rjmp  MAIN      	;Externer Int.0
rjmp  MAIN      	;Externer Int.1
rjmp  MAIN      	;Timer Int.1 event
rjmp  MAIN      	;Timer Int.1 match
rjmp  MAIN      	;Timer Int.1 overflow
rjmp  MAIN      	;Timer Int.0 overflow
rjmp  MAIN      	;UART read
rjmp  MAIN      	;UART empty
rjmp  MAIN      	;UART write
rjmp  MAIN      	;AnalogComp. Int.


main:
	;wdr
   ;ldi	tmp,0x08
   ;out  WDTCR,tmp         ;Watchdog enable 16ms Zyklen
   ldi	tmp,LOW(RAMEND)	;Initiate Stackpointer
	out	SPL,tmp;
	; clear the placeholders for special signs
	clr	SpecialSig1
	clr	SpecialSig2

Init:
	;***************************************************************************
;***     Portbit-Configuration	***

;***	I2CPORT bits
;***		1xxxxxx	   	PD6(Pin11)
;***		x1xxxxx	   	PD5(Pin09)
;***  	xx1xxxx			PD4(Pin08)
;***  	xxx1xxx			PD3(Pin07)
;***  	xxxx1xx	  		PD2(Pin06)
;***  	xxxxx1x 	  		PD1(Pin03)
;***  	xxxxxx1			PD0(Pin02)
;***	PORTB bits
;***		1xxxxxxx		PB7(Pin19)
;***		x1xxxxxx		PB6(Pin18)
;***		xx1xxxxx		PB5(Pin17)
;***     xxx1xxxx	   I2C SCL	PB4(Pin16)
;***     xxxx1xxx	   I2C SDA	PB3(Pin15)
;***     xxxxx1xx	   PB2(Pin14)
;***     xxxxxx1x	   PB1(Pin13)
;***		xxxxxxx1   	PB0(Pin12)


	ldi	tmpstatus,0b00000000	; PD0 and PD1 -> Input, No Pullup
	out	I2CDDR,tmpstatus		; PD2 -> Input, switch on Pullup
	ldi	tmpstatus,0b00000000	; PD3,PD4 -> Output
	out	I2CPORT,tmpstatus		; set TWI pins to open colector
	clr	tmpstatus				; clear TWI status register (used
										; as a temporary register)
	
	;ldi	tmp, $ffg
	;out	ddrd,tmp

MAINLOOP:
	; Example for a message which will be put together by a fixed text
	; and a variable 
	
	; first load the adress of the message in PM
	ldi 	ZH,HIGH(2*Message01) 
	ldi 	ZL,LOW(2*Message01) 
	rcall GetMessage
	ldi	tmp, 28
	rcall bin2bcd8
	ldi	xh,0
	ldi	xl,0x63
	ldi	tmpdata, 48
	add	tmp, tmpdata
	add	tmpdelay, tmpdata	
	st	 	X+, tmpdelay   ; store BCDH to SRAM
	st	 	X, tmp			; store BCDL to SRAM
	
	; call output procedure
	rcall	I2CDISPWM
	
     ldi  R17, $A9
WGLOOP0:  ldi  R18, $92
WGLOOP1:  ldi  R19, $F8
WGLOOP2:  dec  R19
          brne WGLOOP2
          dec  R18
          brne WGLOOP1
          dec  R17
          brne WGLOOP0
			 
	; now put a message to the I2C-Display
	; first load the adress of the message in PM
	ldi 	ZH,HIGH(2*Message02) 
	ldi 	ZL,LOW(2*Message02) 
	; call output procedure
	rcall	I2CDISP
		; ============================= 
;   Warteschleifen-Generator 
;     18432000 Zyklen:
; ----------------------------- 
; warte 18431985 Zyklen:
          ldi  R17, $A9
WGLOOP4:  ldi  R18, $92
WGLOOP5:  ldi  R19, $F8
WGLOOP6:  dec  R19
          brne WGLOOP6
          dec  R18
          brne WGLOOP5
          dec  R17
          brne WGLOOP4

; ============================= 
	
LOOPFOREVER:
	rjmp	MAINLOOP	; Loop Forever

.include "I2CDispR.asm"	
.include "Math.asm"

; Definition of messages
;	Note: Use only capital letters
	
Message01:
.db "IN:    "
Message02:
.db "HALLOWEL"

;*** End of File